import { useState } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var KANA_REGEX = /([ 　ぁあ-んー]+)/g;
var NON_KANA_REGEX = /([^ 　ぁあ-んー]+)/g;
var SPACE_REGEX = /([ 　]+)/g;
var isKana = function (value) { return !!value.match(KANA_REGEX); };
var isNonKana = function (value) { return !!value.match(NON_KANA_REGEX); };
// Split into kanas, spaces, other characters
// Filter out empty values
var splitIntoCharGroups = function (value) {
    return value
        .split(KANA_REGEX)
        .flatMap(function (str) { return str.split(SPACE_REGEX); })
        .filter(Boolean);
};
var extractDiff = function (from, to) {
    var added = to.filter(function (x) { return !from.includes(x); });
    var removed = from.filter(function (x) { return !to.includes(x); });
    return { added: added, removed: removed };
};
var extractPairFromDiff = function (_a) {
    var _b, _c;
    var added = _a.added, removed = _a.removed;
    if (added.length === 1 && removed.length === 1 && isNonKana(added[0]) && isKana(removed[0])) {
        // given
        //   added: ['山田']
        //   removed: ['やまだ']
        // then
        //   paid is { '山田': 'やまだ' }
        return _b = {},
            _b[added[0]] = removed[0],
            _b;
    }
    else if (removed.length === 2 && added.length === 1) {
        // given
        //   added: ['山田']
        //   removed: ['山', 'だ']
        // then
        //   pair is { '田': 'だ' }
        //
        // given
        //   added: ['山田']
        //   removed: ['やま', '田']
        // then
        //   pair is { '山': 'やま' }
        var nonKana = added[0].replace(new RegExp("" + removed.join('|'), 'g'), '');
        var kana = removed.find(isKana);
        return _c = {},
            _c[nonKana] = kana,
            _c;
    }
    else {
        return {};
    }
};
var findPair = function (charGroupsCandidates, currentCharGroups, setLastConvertedCharGroups) {
    var previousCharGroups = charGroupsCandidates[0], tail = charGroupsCandidates.slice(1);
    if (!previousCharGroups) {
        return {};
    }
    var pair = extractPairFromDiff(extractDiff(previousCharGroups, currentCharGroups));
    if (Object.keys(pair).length !== 0) {
        // If a pair of non-kana and kana is found, memoize it for later comparison.
        // This is for a case when a user convert an input from kana to non-kana to non-kana.
        // e.g. 'た' => '多' => '田' . In this case, it needs to store 'た' when making pair of { '多': 'た' }
        //      for the next making pair of { '田': 'た' }
        setLastConvertedCharGroups(previousCharGroups);
        return pair;
    }
    else {
        // The first trial to find pair (between the latest input and the 2nd latest input) fails,
        // it retries once more by using `lastConvertedCharGroups`.
        return findPair(tail, currentCharGroups, setLastConvertedCharGroups);
    }
};
var convertCharGroupsToKana = function (kanaMap, charGroups) {
    var knownNonKanas = Object.keys(kanaMap);
    return charGroups
        .map(function (chars) {
        return knownNonKanas
            .filter(function (knownNonKana) { return chars.indexOf(knownNonKana) >= 0; })
            .reduce(function (memo, nonKana) { return memo.replace(nonKana, kanaMap[nonKana]); }, chars);
    })
        .filter(isKana)
        .join('');
};
var useKana = function () {
    // used by library users
    var _a = useState(''), kana = _a[0], setKana = _a[1];
    // library internal use
    var _b = useState([]), previousCharGroups = _b[0], setPreviousCharGroups = _b[1];
    var _c = useState([]), lastConvertedCharGroups = _c[0], setLastConvertedCharGroups = _c[1];
    var _d = useState({}), kanaMap = _d[0], setKanaMap = _d[1];
    var setKanaSource = function (value) {
        if (value === '') {
            // If a user inputs nothing, reset everything
            setKana('');
            setKanaMap({});
            setPreviousCharGroups([]);
        }
        else {
            var currentCharGroups = splitIntoCharGroups(value);
            // Create kana map that contains pairs of non-kana and kana based on the original map
            var latestKanaMap = __assign(__assign({}, kanaMap), findPair([previousCharGroups, lastConvertedCharGroups], currentCharGroups, setLastConvertedCharGroups));
            var currentKana = convertCharGroupsToKana(latestKanaMap, currentCharGroups);
            setKana(currentKana);
            setKanaMap(latestKanaMap);
            setPreviousCharGroups(currentCharGroups);
        }
    };
    return { kana: kana, setKanaSource: setKanaSource };
};

export { useKana };
